@@ -250,16 +250,14 @@ static int set_option_auo(char const *filename, int line, OPTION *option, char c
 		return -1;
 	}
 
-	if ((iptr = malloc(sizeof(iptr))) == NULL) {
+	if ((iptr = malloc(sizeof(*iptr))) == NULL) {
 			rc_log(LOG_CRIT, "read_config: out of memory");
 			free(p_dupe);
 			return -1;
 	}
 
 	*iptr = 0;
-	/*if(strstr(p_dupe,", \t") != NULL) {*/
-		p_pointer = strtok_r(p_dupe, ", \t", &p_save);
-	/*}*/
+	p_pointer = strtok_r(p_dupe, ", \t", &p_save);
 
 	if (!strncmp(p_pointer, "local", 5))
 			*iptr = AUTH_LOCAL_FST;
@@ -400,34 +398,155 @@ rc_handle *rc_config_init(rc_handle *rh)
 	return rh;
 }
 
+static ssize_t plain_sendto(void *ptr, int sockfd,
+			    const void *buf, size_t len, int flags,
+			    const struct sockaddr *dest_addr, socklen_t addrlen)
+{
+	return sendto(sockfd, buf, len, flags, dest_addr, addrlen);
+}
+
+static ssize_t plain_tcp_sendto(void *ptr, int sockfd,
+			    const void *buf, size_t len, int flags,
+			    const struct sockaddr *dest_addr, socklen_t addrlen)
+{
+	if((connect(sockfd, dest_addr, addrlen)) != 0){
+		rc_log(LOG_ERR, "%s: Connect Call Failed : %s", __FUNCTION__, strerror(errno));
+		return -1;
+	}
+	return sendto(sockfd, buf, len, flags, dest_addr, addrlen);
+}
+
+static ssize_t plain_recvfrom(void *ptr, int sockfd,
+			      void *buf, size_t len, int flags,
+			      struct sockaddr *src_addr, socklen_t * addrlen)
+{
+	return recvfrom(sockfd, buf, len, flags, src_addr, addrlen);
+}
+
+static void plain_close_fd(int fd)
+{
+	close(fd);
+}
+
+static int plain_get_fd(void *ptr, struct sockaddr *our_sockaddr)
+{
+	int sockfd;
+
+	sockfd = socket(our_sockaddr->sa_family, SOCK_DGRAM, 0);
+	if (sockfd < 0) {
+		return -1;
+	}
+
+	if (our_sockaddr->sa_family == AF_INET)
+		((struct sockaddr_in *)our_sockaddr)->sin_port = 0;
+	else
+		((struct sockaddr_in6 *)our_sockaddr)->sin6_port = 0;
+
+	if (bind(sockfd, SA(our_sockaddr), SA_LEN(our_sockaddr)) < 0) {
+		close(sockfd);
+		return -1;
+	}
+	return sockfd;
+}
+
+static int plain_tcp_get_fd(void *ptr, struct sockaddr *our_sockaddr)
+{
+	int sockfd;
+
+	sockfd = socket(our_sockaddr->sa_family, SOCK_STREAM, 0);
+	if (sockfd < 0) {
+		return -1;
+	}
+
+	if (our_sockaddr->sa_family == AF_INET)
+		((struct sockaddr_in *)our_sockaddr)->sin_port = 0;
+	else
+		((struct sockaddr_in6 *)our_sockaddr)->sin6_port = 0;
+
+	if (bind(sockfd, SA(our_sockaddr), SA_LEN(our_sockaddr)) < 0) {
+		close(sockfd);
+		return -1;
+	}
+	return sockfd;
+}
+
+const static rc_sockets_override default_socket_funcs = {
+	.get_fd = plain_get_fd,
+	.close_fd = plain_close_fd,
+	.sendto = plain_sendto,
+	.recvfrom = plain_recvfrom
+};
+
+const static rc_sockets_override default_tcp_socket_funcs = {
+	.get_fd = plain_tcp_get_fd,
+	.close_fd = plain_close_fd,
+	.sendto = plain_tcp_sendto,
+	.recvfrom = plain_recvfrom
+};
+
+static int set_addr(struct sockaddr_storage *ss, const char *ip)
+{
+	memset(ss, 0, sizeof(*ss));
+	if (inet_pton(AF_INET, ip, &((struct sockaddr_in *)ss)->sin_addr) == 1) {
+		ss->ss_family = AF_INET;
+	} else if (inet_pton(AF_INET6, ip, &((struct sockaddr_in6 *)ss)->sin6_addr) == 1) {
+		ss->ss_family = AF_INET6;
+	} else {
+		rc_log(LOG_CRIT, "invalid IP address for nas-ip: %s", ip);
+		return -1;
+	}
+	return 0;
+}
+
 static int apply_config(rc_handle *rh)
 {
+	const char *txt;
+	int ret;
+
 	memset(&rh->own_bind_addr, 0, sizeof(rh->own_bind_addr));
 	rh->own_bind_addr_set = 0;
 	rc_own_bind_addr(rh, &rh->own_bind_addr);
 	rh->own_bind_addr_set = 1;
-#ifdef HAVE_GNUTLS
-        {
-                const char *txt;
-                int ret;
-                txt = rc_conf_str(rh, "serv-auth-type");
-                if (txt != NULL) {
-                        if (strcasecmp(txt, "dtls") == 0) {
-                                ret = rc_init_tls(rh, SEC_FLAG_DTLS);
-                        } else if (strcasecmp(txt, "tls") == 0) {
-                                ret = rc_init_tls(rh, 0);
-                        } else {
-                                rc_log(LOG_CRIT, "unknown server authentication type: %s", txt);
-                                return -1;
-                        }
 
-                        if (ret < 0) {
-                                rc_log(LOG_CRIT, "error initializing %s", txt);
-                                return -1;
-                        }
-                }
-        }
+	txt = rc_conf_str(rh, "nas-ip");
+	if (txt != NULL) {
+		if (set_addr(&rh->nas_addr, txt) < 0)
+			return -1;
+		rh->nas_addr_set = 1;
+	}
+
+	txt = rc_conf_str(rh, "serv-type");
+	if (txt == NULL)
+		txt = rc_conf_str(rh, "serv-auth-type");
+
+	if (txt == NULL)
+		txt = "udp";
+
+	if (strcasecmp(txt, "udp") == 0) {
+		memset(&rh->so, 0, sizeof(rh->so));
+		rh->so_type = RC_SOCKET_UDP;
+		memcpy(&rh->so, &default_socket_funcs, sizeof(rh->so));
+		ret = 0;
+	} else if (strcasecmp(txt, "tcp") == 0) {
+		memset(&rh->so, 0, sizeof(rh->so));
+		rh->so_type = RC_SOCKET_TCP;
+		memcpy(&rh->so, &default_tcp_socket_funcs, sizeof(rh->so));
+		ret = 0;
+#ifdef HAVE_GNUTLS
+	} else if (strcasecmp(txt, "dtls") == 0) {
+		ret = rc_init_tls(rh, SEC_FLAG_DTLS);
+	} else if (strcasecmp(txt, "tls") == 0) {
+		ret = rc_init_tls(rh, 0);
 #endif
+	} else {
+		rc_log(LOG_CRIT, "unknown server type: %s", txt);
+		return -1;
+	}
+
+	if (ret < 0) {
+		rc_log(LOG_CRIT, "error initializing %s", txt);
+		return -1;
+	}
 
 	return 0;
 
@@ -736,7 +855,7 @@ static int find_match (const struct addrinfo* addr, const struct addrinfo *hostn
 static int rc_ipaddr_local(const struct sockaddr *addr)
 {
 	int temp_sock, res, serrno;
-	struct sockaddr tmpaddr;
+	struct sockaddr_storage tmpaddr;
 
 	memcpy(&tmpaddr, addr, SA_LEN(addr));
 
@@ -749,7 +868,7 @@ static int rc_ipaddr_local(const struct sockaddr *addr)
 	} else {
 		((struct sockaddr_in6*)&tmpaddr)->sin6_port = 0;
 	}
-	res = bind(temp_sock, &tmpaddr, SA_LEN(&tmpaddr));
+	res = bind(temp_sock, SA(&tmpaddr), SS_LEN(&tmpaddr));
 	serrno = errno;
 	close(temp_sock);
 	if (res == 0)
@@ -934,7 +1053,10 @@ int rc_find_server_addr (rc_handle const *rh, char const *server_name,
  * rc_config_free:
  * @param rh a handle to parsed configuration
  *
- * Free allocated config values
+ * Free allocated config values. For legacy compatibility
+ * reasons this will not release any dictionary entries.
+ * To release all memory from the handle use rc_destroy()
+ * instead.
  *
  */
 void rc_config_free(rc_handle *rh)
