@@ -504,6 +504,7 @@ void rc_deinit_tls(rc_handle * rh)
 			gnutls_certificate_free_credentials(st->x509_cred);
 		if (st->psk_cred)
 			gnutls_psk_free_client_credentials(st->psk_cred);
+		gnutls_global_deinit();
 	}
 	free(st);
 }
@@ -529,6 +530,14 @@ int rc_init_tls(rc_handle * rh, unsigned flags)
 	char hostname[256];	/* server's hostname */
 	unsigned port;		/* server's port */
 
+	ret = gnutls_global_init();
+	if (ret < 0) {
+		rc_log(LOG_ERR,
+		       "%s: error initializing gnutls: %s",
+		       __func__, gnutls_strerror(ret));
+		return -1;
+	}
+
 	memset(&rh->so, 0, sizeof(rh->so));
 
 	if (flags & SEC_FLAG_DTLS) {
@@ -570,8 +579,8 @@ int rc_init_tls(rc_handle * rh, unsigned flags)
 			if (ret < 0) {
 				ret = -1;
 				rc_log(LOG_ERR,
-				       "%s: error in setting X.509 trust file: %s",
-				       __func__, gnutls_strerror(ret));
+				       "%s: error in setting X.509 trust file: %s: %s",
+				       __func__, gnutls_strerror(ret), ca_file);
 				goto cleanup;
 			}
 		}
@@ -585,8 +594,8 @@ int rc_init_tls(rc_handle * rh, unsigned flags)
 			if (ret < 0) {
 				ret = -1;
 				rc_log(LOG_ERR,
-				       "%s: error in setting X.509 cert and key file: %s",
-				       __func__, gnutls_strerror(ret));
+				       "%s: error in setting X.509 cert and key files: %s: %s - %s",
+				       __func__, gnutls_strerror(ret), cert_file, key_file);
 				goto cleanup;
 			}
 		}
