@@ -17,6 +17,8 @@ size_t rc_strlcpy(char *dst, char const *src, size_t siz);
 
 #include <includes.h>
 
+#define	SA(p)	((struct sockaddr *)(p))
+
 #if !defined(SA_LEN)
 #define SA_LEN(sa) \
   (((sa)->sa_family == AF_INET) ? \
@@ -47,11 +49,23 @@ void rc_own_bind_addr(rc_handle *rh, struct sockaddr_storage *lia);
 double rc_getctime(void);
 void rc_str2tm (char const *valstr, struct tm *tm);
 
+#undef rc_log
+
 #ifdef _MSC_VER /* TODO: Fix me */
 # define rc_log(...)
 #else
-# define rc_log syslog
+
+# ifdef __GNUC__
+#  define rc_log(prio, fmt, ...) \
+	syslog(prio, "radcli: %s: "fmt, __func__, ##__VA_ARGS__)
+# else
+#  define rc_log syslog
+# endif
 #endif
 
+extern unsigned int radcli_debug;
+
+#define		DEBUG(args...)	if(radcli_debug) rc_log(args)
+
 #endif /* UTIL_H */
 
